/**
 * Common types, utility functions.
 * @module data/common
 */
// /** Semantic UI component behavior API */
// export type SemanticCommand = (
//     command: string,
//     v1?: unknown,
//     v2?: unknown,
//     v3?: unknown
// ) => unknown;
// /** Return type for a simple svelte action; with destroy(), but without update(). */
// export type ActionReturnType = {
//     destroy: () => void;
// } | void;
//-----------------------------------------------------------------------------
/*
            dP   oo dP
            88      88
 dP    dP d8888P dP 88
 88    88   88   88 88
 88.  .88   88   88 88
 `88888P'   dP   dP dP

*/
/** Compare two Date objects */
export function equalDates(a1, a2) {
    if (a1 instanceof Date && a2 instanceof Date) {
        return a1.getTime() === a2.getTime();
    }
    return a1 === a2;
}
/** Compare two string arrays or strings */
export function equalStringArrays(a1, a2) {
    if (Array.isArray(a1) && Array.isArray(a2)) {
        if (a1.length !== a2.length) {
            return false;
        }
        for (let i = 0; i < a1.length; i++) {
            if (a1[i] !== a2[i]) {
                return false;
            }
        }
        return true;
    }
    else {
        return a1 === a2;
    }
}
/** Compare two number arrays or numbers */
export function equalNumberArrays(a1, a2) {
    if (Array.isArray(a1) && Array.isArray(a2)) {
        if (a1.length !== a2.length) {
            return false;
        }
        for (let i = 0; i < a1.length; i++) {
            if (a1[i] !== a2[i]) {
                return false;
            }
        }
        return true;
    }
    else {
        return a1 === a2;
    }
}
/** Format a number, padded with "0" to the minimum length */
export function pad(n, size) {
    let str = n.toString();
    while (str.length < size) {
        str = `0${str}`;
    }
    return str;
}
/** Format Date using YYYY-MM-DD format */
export function isoDate(d) {
    if (!d || !d.getDate) {
        return "";
    }
    const day = pad(d.getDate(), 2);
    const month = pad(d.getMonth() + 1, 2);
    const year = pad(d.getFullYear(), 4);
    return `${year}-${month}-${day}`;
}
/** Format Date as time using HH:mm format (24 hours) */
export function isoTime(d) {
    if (!d || !d.getDate) {
        return "";
    }
    const hour = pad(d.getHours(), 2);
    const minute = pad(d.getMinutes(), 2);
    return `${hour}:${minute}`;
}
/** Convert objects to JSON string, for jQuery and DOM elements - get the HTML */
export function stringify(obj) {
    if (!obj || typeof obj !== "object") {
        return `${obj}`;
    }
    const jqueryObj = obj;
    if (jqueryObj.prop && typeof jqueryObj.prop === "function") {
        const html = jqueryObj.prop("outerHTML");
        if (html) {
            return `jQuery: ${html.slice(0, html.indexOf(">") + 1)}`;
        }
    }
    const domObj = obj;
    if (domObj.outerHTML) {
        const html = domObj.outerHTML;
        if (html) {
            return `DOM: ${html.slice(0, html.indexOf(">") + 1)}`;
        }
    }
    // Exclude circular references and '__' props
    const seen = new WeakSet();
    return JSON.stringify(obj, (key, value) => {
        if (typeof value === "object" && value !== null) {
            if (seen.has(value)) {
                return undefined;
            }
            seen.add(value);
        }
        if (key.startsWith("__")) {
            return undefined;
        }
        return value;
    });
}
function noop(..._args) {
    void _args;
}
export class Logger {
    loglevelToNumber(lev) {
        for (let i = 0; i < this.LOG_LEVELS.length; i++) {
            if (this.LOG_LEVELS[i] === lev) {
                return i;
            }
        }
        throw new Error(`Unrecognized log level: ${lev}`);
    }
    constructor(prefix) {
        this.prefix = "";
        this.error = noop;
        this.warn = noop;
        this.log = noop;
        this.info = noop;
        this.debug = noop;
        this.LOG_LEVELS = ["off", "error", "warn", "log", "info", "debug"];
        this.prefix = prefix;
    }
    build(level, fn) {
        const numLevel = this.loglevelToNumber(level);
        for (let i = 1; i < this.LOG_LEVELS.length; i++) {
            const iLevel = this.LOG_LEVELS[i];
            if (numLevel >= i) {
                if (fn) {
                    this[iLevel] = (...args) => {
                        fn(iLevel, this.prefix, ...args);
                    };
                }
                else {
                    this[iLevel] = (...args) => {
                        console[iLevel](this.prefix, ...args);
                    };
                }
            }
            else {
                this[iLevel] = noop;
            }
        }
    }
}
function localTime() {
    const currentDate = new Date();
    const localDate = new Date(currentDate.getTime() - currentDate.getTimezoneOffset() * 60000);
    return localDate.toISOString().slice(0, -1).replace("T", " ");
}
export function buildHttpLogger(logger, level, url) {
    function httpLogImpl(lev, ...args) {
        async function sendHttpLog() {
            const params = {};
            params.time = localTime();
            params.level = lev;
            args.forEach((arg, ind) => {
                params[`msg_${ind}`] = stringify(arg);
            });
            const urlWithParams = `${url}?${new URLSearchParams(params).toString()}`;
            try {
                const response = await fetch(urlWithParams);
                await response.text();
            }
            catch (error) {
                // ignore errors
            }
        }
        // don't wait for promise completion
        void sendHttpLog();
    }
    logger.build(level, httpLogImpl);
}
export const formLog = new Logger("FORM");
export const compLog = new Logger("--");
export const initLog = new Logger(">>>");
