/**
 * DOM search and manipulation, uses jQuery.
 * @module data/dom-jquery
 */
/** Name of the jQuery data attribute used to store form controller */
export const SVELTE_FORM_STORE = "svelte_form_store";
/** List of all init modes, used for runtime check */
const ALL_MODES = ["parent", "child", "sibling"];
/** Global setting for finding corresponding Semantic UI component */
let COMPONENT_INIT_MODES = ["sibling"];
/** How Init* components are looking for a Semantic UI input */
export function getComponentInitMode() {
    return COMPONENT_INIT_MODES;
}
/** Change component finding algorithm.
 *  Setting to all modes will find component in any of the 3 locations.
 *  Setting to empty array makes Init* component work only using forId param. */
export function setComponentInitMode(modes) {
    modes.forEach((mode) => {
        if (!ALL_MODES.includes(mode)) {
            throw new Error(`Invalid component init mode: ${mode}`);
        }
    });
    COMPONENT_INIT_MODES = modes;
}
//-----------------------------------------------------------------------------
/** Global unique id sequence */
let unum = 100;
/** Generate an unique number */
export function nextUid() {
    unum = unum + 1;
    return `00${unum % 1000}`.slice(-3);
}
//-----------------------------------------------------------------------------
/** Find jQuery element by string selector. */
export function jQueryBySelector(selector) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(selector);
}
/** Gets jQuery element for the dom node. */
export function jQueryElem(node) {
    const jQuery = window.jQuery;
    if (!jQuery) {
        throw new Error("jQuery is not initialized");
    }
    return jQuery(node);
}
/** Find corresponding Semantic UI component,
 *  where the `span` Element is a parent, child, or next sibling of the component.
 *  Search mode is usually set in setComponentInitMode(), but may be overridden by `search` parameter. */
export function findComponent(span, selector, id, search) {
    let elem;
    if (id) {
        // use "forId" attribute to find the element to connect to
        elem = jQueryBySelector(`#${id}`);
        if (elem.length && elem.is(selector)) {
            return elem;
        }
        throw new Error(`Can't find '${selector}' component with id="${id}"`);
    }
    const searchModes = search ? search : COMPONENT_INIT_MODES;
    if (searchModes.includes("parent")) {
        elem = jQueryElem(span).find(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    if (searchModes.includes("child")) {
        elem = jQueryElem(span).parent(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    if (searchModes.includes("sibling")) {
        elem = jQueryElem(span).prev(selector);
        if (elem.length > 0) {
            return elem;
        }
    }
    throw new Error(`Can't find '${selector}' component as a ${searchModes.join("|")}`);
}
//-----------------------------------------------------------------------------
/** Get field identifier: id, name, data-validate */
export function getFieldKey(elem) {
    if (!elem) {
        return undefined;
    }
    return elem.attr("id") ?? elem.attr("name") ?? elem.attr("data-validate");
}
/** Get or assign sequential field identifier: id, name, data-validate */
export function getOrAssignKey(elem, prefix) {
    let key = getFieldKey(elem);
    if (!key) {
        // assign new attribute
        key = `${prefix}_${nextUid()}`;
        elem.attr("data-validate", key);
    }
    return key;
}
/** If the elem doesn't have key, copy parent field identifier: id, name, data-validate; or assign a unique id. */
export function copyParentKey(elem, parent, prefix) {
    let key = getFieldKey(elem);
    if (!key) {
        // assign new attribute
        const parentKey = getFieldKey(parent);
        key = `${prefix}_${parentKey ?? nextUid()}`;
        elem.attr("data-validate", key);
    }
    return key;
}
/** If the elem doesn't have key, copy parent field identifier: id, name, data-validate.
 *  But only if the parent key exists. */
function copyKeyIfExists(elem, parent, prefix) {
    let key = getFieldKey(elem);
    if (!key) {
        // get parent key
        const parentKey = getFieldKey(parent);
        if (parentKey) {
            key = `${prefix}_${parentKey}`;
            elem.attr("data-validate", key);
        }
    }
    return key;
}
/** Perform lookup for known parent types or .field with id, copy parent id if it can. */
export function ensureFieldKey(elem) {
    const key = getFieldKey(elem);
    if (key) {
        return key;
    }
    let parent = elem.parent();
    while (parent && parent.length > 0) {
        if (parent.is(".ui.calendar")) {
            const k = copyKeyIfExists(elem, parent, "f_calendar");
            if (k)
                return k;
        }
        else if (parent.is(".ui.dropdown")) {
            const k = copyKeyIfExists(elem, parent, "f_dropdown");
            if (k)
                return k;
        }
        else if (parent.is(".ui.checkbox")) {
            const k = copyKeyIfExists(elem, parent, "f_checkbox");
            if (k)
                return k;
        }
        else if (parent.is(".ui.slider")) {
            const k = copyKeyIfExists(elem, parent, "f_slider");
            if (k)
                return k;
        }
        if (parent.is(".field")) {
            const k = copyKeyIfExists(elem, parent, "f_field");
            if (k)
                return k;
            break; // while
        }
        if (parent.is("form")) {
            break; // while
        }
        // continue going up in DOM hierarchy
        parent = parent.parent();
    }
    // no key where found, assign new attribute
    return getOrAssignKey(elem, "f");
}
//-----------------------------------------------------------------------------
/** Search DOM tree for a parent form element  */
export function findParentForm(elem) {
    const form = elem.closest("form.ui.form");
    if (form.length) {
        return form;
    }
    else {
        return undefined;
    }
}
/** Find the label in the the same .ui.field with for="_" */
export function findLabelWithBlank(node) {
    const field = node.parent().filter(".field");
    const label = field.find("label");
    if (label.attr("for") === "_") {
        return label;
    }
    else {
        return undefined;
    }
}
