/**
 * Svelte field validation controller, used by all Init Svelte components.
 * @module data/field-controller
 */
import { findParentForm, SVELTE_FORM_STORE, ensureFieldKey } from "../data/dom-jquery";
/** Adds validation rule to the field.
 *  Common class used by all Init* components to control field validation */
export class FieldController {
    /** Create ne field controller for the input. If it is validated, find and store form controller */
    constructor(type, input, validationRules) {
        this.type = type;
        this.key = ensureFieldKey(input);
        // get parent form and form controller
        if (validationRules) {
            const form = findParentForm(input);
            if (!form) {
                throw new Error(`Validated field ${this.key} must be a child of a <form class="ui form">`);
            }
            this.formCtrl = form.data(SVELTE_FORM_STORE);
            if (!this.formCtrl) {
                throw new Error(`Form controller for ${this.key} is not initialized, use <InitForm> on the parent form element`);
            }
            this.formCtrl.addRule(this.key, validationRules);
            this.rules = validationRules;
            // revalidate after the form has updated it's definition
            void this.revalidate(); // NEW
        }
    }
    /** Update filed validation rules if they are changed at runtime */
    replaceRules(validationRules) {
        function isEmpty(val) {
            return val === undefined || val === "" || (Array.isArray(val) && val.length === 0);
        }
        if (isEmpty(this.rules) && !isEmpty(validationRules)) {
            // No rules previously, now we have rules
            this.rules = validationRules;
            this.formCtrl?.addRule(this.key, this.rules);
            void this.revalidate();
            return;
        }
        else if (!isEmpty(this.rules) && isEmpty(validationRules)) {
            // Replace rules we had with nothing
            this.rules = validationRules;
            this.removeRules();
            void this.revalidate();
            return;
        }
        function isRuleObj(val) {
            return typeof val === "object" && "type" in val;
        }
        function isEqual(lhs, rhs) {
            if (lhs === rhs) {
                return true;
            }
            else if (isRuleObj(lhs) && isRuleObj(rhs)) {
                return lhs.type === rhs.type && lhs.prompt === rhs.prompt;
            }
            else if (Array.isArray(lhs) && Array.isArray(rhs)) {
                if (lhs.length === rhs.length) {
                    for (let i = 0, len = lhs.length; i !== len; ++i) {
                        if (!isEqual(lhs[i], rhs[i])) {
                            return false;
                        }
                    }
                    return true;
                }
                return false;
            }
            return false;
        }
        if (!isEqual(this.rules, validationRules)) {
            this.formCtrl?.removeRule(this.key, this.rules);
            this.rules = validationRules;
            this.formCtrl?.addRule(this.key, this.rules);
            void this.revalidate();
        }
    }
    // /** Simpler replaceRules function, assumes that it is called only when rules change */
    // replaceRules(validationRules?: RuleDefinition): void {
    //     if (this.rules) {
    //         this.formCtrl?.removeRule(this.key, this.rules);
    //         this.rules = undefined;
    //     }
    //     if (validationRules) {
    //         this.formCtrl?.addRule(this.key, validationRules);
    //         this.rules = validationRules;
    //     }
    //     void this.revalidate();
    // }
    /** Validate the new field value, if the field is validated (has form controller).
     *  This method is `async` as it debounces (deduplicates) the validation event
     *  in case multiple fields are modified in a single Svelte 'tick'. */
    async revalidate() {
        if (this.formCtrl) {
            await this.formCtrl.revalidateField(this.key);
        }
    }
    /** Remove rules and revalidate */
    removeRules() {
        if (this.formCtrl && this.rules) {
            this.formCtrl.removeRule(this.key, this.rules);
            // revalidate after the form has updated it's definition
            void this.revalidate();
        }
    }
}
