/**
 * Svelte form validation controller, used by InitForm, implements FormController interface.
 * @module data/form-controller-impl
 */
import type { FormSettings, JQueryApi, RuleDefinition } from "./semantic-types";
import type { FormController } from "./form-controller";
/** Semantic UI form behavior */
export type FormApi = {
    form(settings?: FormSettings): void;
    form(command: "add rule", key: string, rule: RuleDefinition): void;
    form(command: "remove field", key: string): void;
    form(command: "validate field", key: string): void;
    form(command: "validate form"): void;
    form(command: "is valid"): boolean;
    form(command: "destroy"): void;
    form(command: "get field", key: string): JQueryApi;
    form(command: "set as clean"): void;
    form(command: "reset"): void;
};
/** Form validation controller. Is accessed only from `<InitForm>` component. */
export declare class FormControllerImpl implements FormController {
    /** Form identifier for debug purposes */
    private formId;
    /** jQuery form element */
    private elem;
    /** Validate form on each field change */
    private active;
    /** Form doesn't validate empty fields */
    private ignoreEmpty;
    /** Map of field validation rules */
    private rules;
    /** Map of field where the validation rules are currently deactivated because the field is empty.
     *  Each rule from the `rules` is either activated or listed in `ignoredFields` */
    private ignoredFields;
    /** Form validation has been triggered by markForValidation(), but not yet performed by validateIfMarked() */
    private mustValidate;
    /** Create new form controller, is used from InitForm. */
    constructor(elem: FormApi, formId: string, active: boolean, ignoreEmpty: boolean);
    /** Make the rule active in Semantic UI */
    private activateRule;
    /** Make the rule inactive in Semantic UI */
    private deactivateRule;
    /** Check if the field value is empty */
    private fieldIsEmpty;
    /** Perform (deduped) form validation */
    private revalidateForm;
    /** Flag the form as requiring validation; used to dedupe multiple form validation calls */
    private markForValidation;
    /** Perform form validation, if marked; used to dedupe multiple validation calls.
     *  Returns false if the validation has already been performed from another async call. */
    private validateIfMarked;
    /** Returns true if the form is revalidated on each field change */
    isActive(): boolean;
    /** Set active flag, ensure that form is validated or reset according to the flag */
    setActive(newValue: boolean): void;
    /** Returns true if validation rules are deactivated for empty fields */
    isIgnoreEmpty(): boolean;
    /** If field is empty, deactivate the filed rules and store in ignoredFields.
     *  Changing 'ignoreEmpty' causes one-time form validation, similar to doValidateForm().
     *  To avoid side effects call setIgnoreEmpty() before calling setActive(false). */
    setIgnoreEmpty(newValue: boolean): void;
    /** Register the field validation rule, activate the rule if the form validation is active */
    addRule(key: string, rule: RuleDefinition): void;
    /** Remove the field validation rule */
    removeRule(key: string, rule: RuleDefinition): void;
    /** Modify the rules if `ignoreEmpty` and field value has changed from or to 'empty'.
     *  If `active` - perform (deduped) form validation, otherwise - validate one field. */
    revalidateField(key: string): Promise<void>;
    /** Trigger validation of the field */
    doValidateField(key: string): void;
    /** Check form validation rules; if necessary, update the UI with error prompts */
    doValidateForm(): void;
    /** Clear 'dirty' flag and sets current values as defaults.
     *  If active == false, removes error messages, which may have appeared when form rules were added or changed. */
    doResetForm(): void;
}
