/**
 * Svelte form validation controller, used by InitForm, implements FormController interface.
 * @module data/form-controller-impl
 */
import { tick } from "svelte";
import { formLog } from "./common";
function ruleToStr(rule) {
    const isObject = Array.isArray(rule) || typeof rule === "object";
    return isObject ? JSON.stringify(rule) : String(rule);
}
/** Form validation controller. Is accessed only from `<InitForm>` component. */
export class FormControllerImpl {
    /*
                      oo                     dP
                                             88
    88d888b. 88d888b. dP dP   .dP .d8888b. d8888P .d8888b.
    88'  `88 88'  `88 88 88   d8' 88'  `88   88   88ooood8
    88.  .88 88       88 88 .88'  88.  .88   88   88.  ...
    88Y888P' dP       dP 8888P'   `88888P8   dP   `88888P'
    88
    dP

    */
    /** Create new form controller, is used from InitForm. */
    constructor(elem, formId, active, ignoreEmpty) {
        /** Map of field validation rules */
        this.rules = {};
        /** Map of field where the validation rules are currently deactivated because the field is empty.
         *  Each rule from the `rules` is either activated or listed in `ignoredFields` */
        this.ignoredFields = {};
        /** Form validation has been triggered by markForValidation(), but not yet performed by validateIfMarked() */
        this.mustValidate = false;
        this.elem = elem;
        this.formId = formId;
        this.active = active;
        this.ignoreEmpty = ignoreEmpty;
    }
    /** Make the rule active in Semantic UI */
    activateRule(key, rule) {
        formLog.log(`(${this.formId}) : add_rule - ${key} : ${ruleToStr(rule)}`);
        this.elem.form("add rule", key, rule);
    }
    /** Make the rule inactive in Semantic UI */
    deactivateRule(key, rule) {
        formLog.log(`(${this.formId}) : remove_rule - ${key} : ${ruleToStr(rule)}`);
        // must use 'remove field', not 'remove rule' if all the rules are removed
        this.elem.form("remove field", key);
    }
    /** Check if the field value is empty */
    fieldIsEmpty(key) {
        const field = this.elem.form("get field", key);
        if (field.attr("type") === "checkbox") {
            const checked = field.attr("checked");
            return !checked;
        }
        // FIXME: deal with radio or checkbox group
        const value = field.val();
        if (Array.isArray(value)) {
            return value.length === 0;
        }
        return !value;
    }
    /** Perform (deduped) form validation */
    async revalidateForm() {
        if (this.active) {
            this.markForValidation("_form_");
            await tick();
            this.validateIfMarked();
        }
        else {
            formLog.info(`(${this.formId}) : skip form revalidation`);
        }
    }
    /** Flag the form as requiring validation; used to dedupe multiple form validation calls */
    markForValidation(key) {
        formLog.debug(`(${this.formId}) : mark (${key})`);
        this.mustValidate = true;
    }
    /** Perform form validation, if marked; used to dedupe multiple validation calls.
     *  Returns false if the validation has already been performed from another async call. */
    validateIfMarked() {
        if (this.mustValidate) {
            formLog.debug(`(${this.formId}) : validate`);
            this.doValidateForm();
            this.mustValidate = false;
            return true;
        }
        else {
            return false;
        }
    }
    /*

    oo                     dP
                           88
    dP 88d8b.d8b. 88d888b. 88
    88 88'`88'`88 88'  `88 88
    88 88  88  88 88.  .88 88
    dP dP  dP  dP 88Y888P' dP
                    88
                    dP
    */
    // used from InitForm.svelte
    /** Returns true if the form is revalidated on each field change */
    isActive() {
        return this.active;
    }
    /** Set active flag, ensure that form is validated or reset according to the flag */
    setActive(newValue) {
        formLog.log(`(${this.formId}) : validate -> ${newValue}`);
        if (newValue === true) {
            // live form validation, updating read-only `valid` and `error` bindings
            this.active = true;
            void this.revalidateForm();
        }
        else {
            // remove validation prompts and switch into validate-as-touched mode
            this.active = false;
            this.doResetForm();
        }
    }
    /** Returns true if validation rules are deactivated for empty fields */
    isIgnoreEmpty() {
        return this.ignoreEmpty;
    }
    /** If field is empty, deactivate the filed rules and store in ignoredFields.
     *  Changing 'ignoreEmpty' causes one-time form validation, similar to doValidateForm().
     *  To avoid side effects call setIgnoreEmpty() before calling setActive(false). */
    setIgnoreEmpty(newValue) {
        formLog.log(`(${this.formId}) : ignore -> ${newValue}`);
        if (newValue === true) {
            // remove rules from empty fields
            Object.keys(this.rules).forEach((key) => {
                if (this.fieldIsEmpty(key)) {
                    this.deactivateRule(key, this.rules[key]);
                    this.ignoredFields[key] = true;
                    // remove error messages from touched empty fields
                    this.doValidateField(key);
                }
            });
            this.ignoreEmpty = true;
            if (this.active) {
                void this.revalidateForm();
            }
        }
        else {
            // reactivate all validation rules
            Object.keys(this.ignoredFields).forEach((key) => {
                this.activateRule(key, this.rules[key]);
            });
            this.ignoredFields = {};
            this.ignoreEmpty = false;
            if (this.active) {
                void this.revalidateForm();
            }
        }
    }
    /*
                      dP       dP oo
                      88       88
    88d888b. dP    dP 88d888b. 88 dP .d8888b.
    88'  `88 88    88 88'  `88 88 88 88'  `""
    88.  .88 88.  .88 88.  .88 88 88 88.  ...
    88Y888P' `88888P' 88Y8888' dP dP `88888P'
    88
    dP

    */
    // accessible from Field Controller and static functions
    /** Register the field validation rule, activate the rule if the form validation is active */
    addRule(key, rule) {
        this.rules[key] = rule;
        if (this.ignoreEmpty && this.fieldIsEmpty(key)) {
            this.ignoredFields[key] = true;
        }
        else {
            this.activateRule(key, rule);
        }
    }
    /** Remove the field validation rule */
    removeRule(key, rule) {
        delete this.rules[key];
        delete this.ignoredFields[key];
        this.deactivateRule(key, rule);
    }
    /** Modify the rules if `ignoreEmpty` and field value has changed from or to 'empty'.
     *  If `active` - perform (deduped) form validation, otherwise - validate one field. */
    async revalidateField(key) {
        if (this.ignoreEmpty && this.rules[key]) {
            // remove form rules if field became empty
            if (this.fieldIsEmpty(key) && !this.ignoredFields[key]) {
                this.deactivateRule(key, this.rules[key]);
                this.ignoredFields[key] = true;
            }
            // add form rules if field became not-empty
            if (!this.fieldIsEmpty(key) && this.ignoredFields[key]) {
                this.activateRule(key, this.rules[key]);
                delete this.ignoredFields[key];
            }
        }
        if (this.active) {
            this.markForValidation(key);
            await tick();
            this.doValidateField(key);
            this.validateIfMarked();
        }
        else {
            this.doValidateField(key);
        }
        // doValidateField() looks redundant for active, but helps with updating UI after formatter,
        // it appears that 'revalidate' option somehow triggers extra field validation with old (empty) rule
    }
    //-------------------------------------------------------------------------
    /** Trigger validation of the field */
    doValidateField(key) {
        this.elem.form("validate field", key);
    }
    /** Check form validation rules; if necessary, update the UI with error prompts */
    doValidateForm() {
        this.elem.form("validate form");
    }
    /** Clear 'dirty' flag and sets current values as defaults.
     *  If active == false, removes error messages, which may have appeared when form rules were added or changed. */
    doResetForm() {
        this.elem.form("set as clean");
        if (!this.active) {
            this.elem.form("reset");
            // manually clear errors messages from the calendar component
            this.elem
                .find(".field")
                .removeClass("error")
                .find(".prompt")
                .remove();
        }
    }
}
