/**
 * Svelte form validation interface, used by FieldController and available to static functions.
 * @module data/form-controller
 */
import type { RuleDefinition } from "./semantic-types";
/** FormController exposes Semantic UI form element and it's data validation.
 *  Hides implementation details of SuiFormController from the FieldController. */
export interface FormController {
    /** Register the field validation rule, activate the rule if the form validation is active */
    addRule: (key: string, rules: RuleDefinition) => void;
    /** Remove the field validation rule */
    removeRule: (key: string, rules: RuleDefinition) => void;
    /** If form validation is active - perform deduped form validation, otherwise - validate one field.
     *  Modify the rules if `ignoreEmpty` == true and field value has changed from or to 'empty'. */
    revalidateField: (key: string) => Promise<void>;
    /** Trigger validation of the individual field.
     *  Form validation is not necessary triggered at the same time. */
    doValidateField: (key: string) => void;
    /** Check form validation rules; update the UI with error prompts.
     *  If InitForm validateForm == true, doValidateForm() becomes redundant. */
    doValidateForm: () => void;
    /** Clear 'dirty' flag, and set current values as defaults.
     *  Is usually called after form data is loaded.
     *
     *  If InitForm validateForm == false,
     *  doResetForm() removes error messages and triggers "validate-on-touch" behavior. */
    doResetForm(): void;
}
/** Find form controller based on event target, selector, or DOM Element of the form or its child */
export declare function getFormController(e: string | Element): FormController;
/** Imperatively call form validation.
 *  The function may be used by name in event handler from within the form - it will find the form using event target.
 *  Alternatively it may accept jQuery selector or DOM Element as a parameter. */
export declare function doValidateForm(e: Event | string | Element): void;
/** Imperatively initialize form state, mark it as clean and, if not active (validateForm), remove validation errors.
 *  The function may be used by name in event handler from within the form - it will find the form using event target.
 *  Alternatively it may accept jQuery selector or DOM Element as a parameter. */
export declare function doResetForm(e: Event | string | Element): void;
