/**
 * Svelte form validation interface, used by FieldController and available to static functions.
 * @module data/form-controller
 */
import { stringify } from "./common";
import { findParentForm, SVELTE_FORM_STORE, jQueryElem, jQueryBySelector } from "./dom-jquery";
/*
 .8888b                              dP   oo
 88   "                              88
 88aaa  dP    dP 88d888b. .d8888b. d8888P dP .d8888b. 88d888b. .d8888b.
 88     88    88 88'  `88 88'  `""   88   88 88'  `88 88'  `88 Y8ooooo.
 88     88.  .88 88    88 88.  ...   88   88 88.  .88 88    88       88
 dP     `88888P' dP    dP `88888P'   dP   dP `88888P' dP    dP `88888P'

*/
/** Find form controller based on event target, selector, or DOM Element of the form or its child */
export function getFormController(e) {
    if (!e) {
        throw new Error("Form not found: getFormController() function requires a parameter");
    }
    let elem = undefined;
    if (typeof e === "string") {
        elem = jQueryBySelector(e);
    }
    else if (e instanceof Element) {
        elem = jQueryElem(e);
    }
    if (!elem) {
        throw new Error(`Form not found: can't find element for ${stringify(e)}`);
    }
    else if (elem.length > 1) {
        throw new Error(`Form not found: there are multiple element for ${stringify(e)}`);
    }
    const form = findParentForm(elem);
    if (!form) {
        throw new Error(`Form not found for ${stringify(elem)}, ensure it has '.ui.form' class`);
    }
    const ctrl = form.data(SVELTE_FORM_STORE);
    if (!ctrl) {
        throw new Error("Sematic UI Form is not initialized, use InitForm");
    }
    return ctrl;
}
/** Imperatively call form validation.
 *  The function may be used by name in event handler from within the form - it will find the form using event target.
 *  Alternatively it may accept jQuery selector or DOM Element as a parameter. */
export function doValidateForm(e) {
    if (!e) {
        throw new Error("validateForm() requires a parameter: event, selector, or Element");
    }
    const e2 = e instanceof Event ? e.target : e;
    const ctrl = getFormController(e2);
    ctrl.doValidateForm();
}
/** Imperatively initialize form state, mark it as clean and, if not active (validateForm), remove validation errors.
 *  The function may be used by name in event handler from within the form - it will find the form using event target.
 *  Alternatively it may accept jQuery selector or DOM Element as a parameter. */
export function doResetForm(e) {
    if (!e) {
        throw new Error("setFormAsClean() requires a parameter: event, selector, or Element");
    }
    const e2 = e instanceof Event ? e.target : e;
    const ctrl = getFormController(e2);
    ctrl.doResetForm();
}
