/**
 * Global `fmt` and `parse` objects with default formatting settings, autocomplete helper for validation rules.
 * @module data/helpers
 */
import { DateFmt } from "../data/format-date";
import { isoDate } from "./common";
import { NumberFmt } from "./format-number";
/*
 .8888b              dP
 88   "              88
 88aaa  88d8b.d8b. d8888P
 88     88'`88'`88   88
 88     88  88  88   88
 dP     dP  dP  dP   dP

*/
// The formatter objects are lazily initialized, to give user the chance to change defaults.
// 'int' differs from 'num' by not using thousand separator.
let dateFmt = null;
let timeFmt = null;
let intFmt = null;
let numFmt = null;
let num2Fmt = null;
let num6Fmt = null;
let moneyFmt = null;
/** Default formatter helper. Intended to be used from code of svelte components:
 * ```
 * Total: {fmt.money(value)}
 * ``` */
export const fmt = {
    date: (d) => {
        dateFmt ?? (dateFmt = new DateFmt({ type: "date" }));
        return dateFmt.format(d);
    },
    time: (d) => {
        timeFmt ?? (timeFmt = new DateFmt({ type: "time" }));
        return timeFmt.format(d);
    },
    int: (n) => {
        intFmt ?? (intFmt = new NumberFmt({ type: "integer" }));
        return intFmt.format(n);
    },
    num: (n) => {
        numFmt ?? (numFmt = new NumberFmt({ type: "decimal" }));
        return numFmt.format(n);
    },
    num2: (n) => {
        num2Fmt ?? (num2Fmt = new NumberFmt({ type: "decimal", precision: 2 }));
        return num2Fmt.format(n);
    },
    num6: (n) => {
        num6Fmt ?? (num6Fmt = new NumberFmt({ type: "decimal", precision: 6 }));
        return num6Fmt.format(n);
    },
    money: (n) => {
        moneyFmt ?? (moneyFmt = new NumberFmt({ type: "money" }));
        return moneyFmt.format(n);
    },
};
/*

 88d888b. .d8888b. 88d888b. .d8888b. .d8888b.
 88'  `88 88'  `88 88'  `88 Y8ooooo. 88ooood8
 88.  .88 88.  .88 88             88 88.  ...
 88Y888P' `88888P8 dP       `88888P' `88888P'
 88
 dP
*/
/** Default parser helper. Intended to be used from code. */
export const parse = {
    date: (s) => {
        dateFmt ?? (dateFmt = new DateFmt({ type: "date" }));
        return dateFmt.parse(s);
    },
    time: (s) => {
        timeFmt ?? (timeFmt = new DateFmt({ type: "time" }));
        return timeFmt.parse(s);
    },
    int: (s) => {
        intFmt ?? (intFmt = new NumberFmt({ type: "integer" }));
        return intFmt.parse(s);
    },
    num: (s) => {
        numFmt ?? (numFmt = new NumberFmt({ type: "decimal" }));
        return numFmt.parse(s);
    },
    num2: (s) => {
        num2Fmt ?? (num2Fmt = new NumberFmt({ type: "decimal", precision: 2 }));
        return num2Fmt.parse(s);
    },
    num6: (s) => {
        num6Fmt ?? (num6Fmt = new NumberFmt({ type: "decimal", precision: 6 }));
        return num6Fmt.parse(s);
    },
    money: (s) => {
        moneyFmt ?? (moneyFmt = new NumberFmt({ type: "money" }));
        return moneyFmt.parse(s);
    },
};
/*
                   dP             dP                dP
                   88             88                88
 88d888b. dP    dP 88 .d8888b.    88d888b. .d8888b. 88 88d888b. .d8888b. 88d888b.
 88'  `88 88    88 88 88ooood8    88'  `88 88ooood8 88 88'  `88 88ooood8 88'  `88
 88       88.  .88 88 88.  ...    88    88 88.  ... 88 88.  .88 88.  ... 88
 dP       `88888P' dP `88888P'    dP    dP `88888P' dP 88Y888P' `88888P' dP
                                                       88
                                                       dP
*/
/** Autocomplete helper for validation rules. */
/* prettier-ignore */
// eslint-disable-next-line @typescript-eslint/typedef
export const rule = {
    /* eslint-disable key-spacing */
    empty: () => "empty",
    checked: () => "checked",
    email: () => "email",
    url: () => "url",
    integer: () => "integer",
    decimal: () => "decimal",
    number: () => "number",
    creditCard: () => "creditCard",
    regex: (reg) => `regExp[//${reg}//]`,
    // integer:    (min?: number, max?: number): string =>
    //     min == undefined && max == undefined ? "integer" : `integer[${min}..${max}]`,
    is: (val) => `is[${val}]`,
    isExactly: (val) => `isExactly[${val}]`,
    not: (val) => `not[${val}]`,
    notExactly: (val) => `notExactly[${val}]`,
    contains: (val) => `contains[${val}]`,
    containsExactly: (val) => `containsExactly[${val}]`,
    doesntContain: (val) => `doesntContain[${val}]`,
    doesntContainExactly: (val) => `doesntContainExactly[${val}]`,
    match: (fld) => `match[${fld}]`,
    different: (fld) => `different[${fld}]`,
    minLength: (n) => `minLength[${n}]`,
    exactLength: (n) => `exactLength[${n}]`,
    maxLength: (n) => `maxLength[${n}]`,
    minCount: (n) => `minCount[${n}]`,
    exactCount: (n) => `exactCount[${n}]`,
    maxCount: (n) => `maxCount[${n}]`,
    // helpers for custom rules, would be better in validation-rules.ts, but are left here for auto-completion
    start: (val) => `start[${val}]`,
    isoDate: () => "isoDate",
    wrappedIn: (val) => `wrappedIn[${val}]`,
    range: (min, max) => `range[${min}..${max}]`,
    size: (min, max) => `size[${min}..${max}]`,
    maxValue: (val) => ((val instanceof Date) ? `maxValue[${isoDate(val)}]` : `maxValue[${val}]`),
    minValue: (val) => ((val instanceof Date) ? `minValue[${isoDate(val)}]` : `minValue[${val}]`),
    /* eslint-enable */
};
