/**
 * Semantic UI types.
 *
 * Re-exports Semantic UI types from the module instead of namespaces.
 *
 * Fixes type definition inconsistencies.
 * @module data/semantic-types
 */
/// <reference types="jquery" />
/// <reference types="fomantic-ui-css" />
/* <reference> lines above were added manually to fix Svelte 5 packaging bug
 */
export type JQueryApi = JQuery<HTMLElement>;
export type DateFormatFn = (d: Date | undefined, settings?: CalendarSettings) => string;
export type DateParseFn = (date: string | undefined, settings?: CalendarSettings) => Date | undefined;
/**
 * Fomantic UI Calendar settings.
 * Adds 'formatter' and 'parser' documentation and fixes 'onChange' arguments.
 *
 * @see {@link https://fomantic-ui.com/modules/calendar.html#/settings}
 */
export type CalendarSettings = Omit<Partial<FomanticUI.CalendarSettings>, "onChange"> & {
    /** Is called after a calendar date has changed. */
    onChange?(this: JQuery<HTMLElement>, date: Date, text: string, mode: string): void;
    formatter?: {
        cellTime?: string | DateFormatFn;
        date?: string | DateFormatFn;
        datetime?: string | DateFormatFn;
        time?: string | DateFormatFn;
        month?: string | DateFormatFn;
        year?: string | DateFormatFn;
        dayHeader?: string | DateFormatFn;
        hourHeader?: string | DateFormatFn;
        minuteHeader?: string | DateFormatFn;
        monthHeader?: string | DateFormatFn;
        yearHeader?: (date: Date, settings: CalendarSettings) => string;
        cell?: (cell: JQuery, date: Date, options: {
            mode: string;
            adjacent: boolean;
            disabled: boolean;
            active: boolean;
            today: boolean;
        }) => string;
        dayColumnHeader?: (day: number, settings: CalendarSettings) => string;
        today?: (settings: CalendarSettings) => string;
    };
    parser?: {
        date?: DateParseFn;
    };
};
export type CalendarText = FomanticUI.Calendar.Settings.Texts;
/** Validation rule object: rule string and custom error prompt */
export type RuleObj = {
    type: string;
    prompt?: string;
};
/** Rule definition takes array or single instance of string or RuleObj */
export type RuleDefinition = string | string[] | RuleObj | RuleObj[];
type SuiValue = string | boolean | number | Date;
/** @see {@link https://fomantic-ui.com/behaviors/form.html#behaviors} */
type SuiModule = {
    clear: () => void;
    determine: {
        isValid: () => boolean;
        isDirty: () => boolean;
    };
    get: {
        field: (identifier: string, strict: boolean) => JQueryApi;
        fields: (identifiers: string[], strict: boolean) => JQueryApi;
        fieldLabel: (identifier: string, useIdAsFallback: boolean) => string;
        value: (identifiers: string, strict: boolean) => SuiValue | SuiValue[] | undefined;
        values: (identifiers: string[], strict: boolean) => Record<string, SuiValue | SuiValue[]>;
        dirtyFields: () => JQueryApi;
    };
    has: {
        field: (identifier: string) => boolean;
    };
    is: {
        blank: (field: JQueryApi) => boolean;
        empty: (field: JQueryApi) => boolean;
        valid: (() => boolean) | ((fieldName: string, showErrors?: boolean) => boolean);
        clean: () => boolean;
        dirty: () => boolean;
        fieldDirty: (field: JQueryApi) => boolean;
        checkboxDirty: (field: JQueryApi) => boolean;
        justDirty: () => boolean;
        justClean: () => boolean;
    };
    reset: () => void;
};
/** Custom validation function */
export type RuleFunc = (value: string, ruleValue: string, module: SuiModule) => boolean;
/**
 * Fomantic UI form validation behavior.
 * Adds 'fields' and 'rules'
 *
 * @see {@link https://fomantic-ui.com/behaviors/form.html#/settings}
 */
export type FormSettings = Partial<FomanticUI.FormSettings> & {
    fields?: Record<string, RuleDefinition>;
    rules?: Record<string, RuleFunc>;
};
export type FormPrompt = Omit<FomanticUI.Form.Settings.Prompts, "contain" | "containExactly"> & {
    contains: string;
    containsExactly: string;
    start: string;
    isoDate: string;
    wrappedIn: string;
    maxValue?: string;
    minValue?: string;
    range?: string;
    size?: string;
};
export type FormText = FomanticUI.Form.Settings.Texts;
/**
 * Dropdown initialization settings.
 * Fixes onChange signature to allow string[] for multi-select
 *
 * @see {@link http://fomantic-ui.com/modules/dropdown.html#/settings}
 */
export type DropdownSettings = Omit<Partial<FomanticUI.DropdownSettings>, "onChange"> & {
    /** Is called after a dropdown value changes.
     *  Receives the name and value of selection and the active menu element. */
    onChange?(value: string | string[], text: string, $choice: JQuery): void;
};
export type DropdownMessages = FomanticUI.Dropdown.Settings.Messages;
/** @see {@link https://fomantic-ui.com/modules/progress.html#/settings} */
export type ProgressSettings = Partial<FomanticUI.ProgressSettings>;
/**
 * Text content for each state, uses simple templating with {percent}, {value}, {total} and {bar}.
 *
 * {bar} is useful to show bar names on multiple bars.
 * Names of bars are provided by text.bars in the forms of array of string. E.g. ['bar1', 'bar2'].
 */
export type ProgressTexts = Partial<FomanticUI.Progress.Settings.Texts>;
export type FlyoutTexts = {
    ok: string;
    cancel: string;
    close: string;
};
/** @see {@link https://fomantic-ui.com/modules/flyout.html#/settings} */
export type FlyoutSettings = Partial<FomanticUI.FlyoutSettings> & {
    text: FlyoutTexts;
};
/** @see {@link https://fomantic-ui.com/modules/checkbox.html#/settings} */
export type CheckboxSettings = Partial<FomanticUI.CheckboxSettings>;
/** @see {@link https://fomantic-ui.com/modules/modal.html#/settings} */
export type ModalSettings = Partial<FomanticUI.ModalSettings>;
/** @see {@link https://fomantic-ui.com/modules/popup.html#/settings} */
export type PopupSettings = Partial<FomanticUI.PopupSettings>;
/** @see {@link https://fomantic-ui.com/modules/slider.html#/settings} */
export type SliderSettings = Partial<FomanticUI.SliderSettings>;
/** @see {@link https://fomantic-ui.com/modules/sticky.html#/settings} */
export type StickySettings = Partial<FomanticUI.StickySettings>;
/** @see {@link https://fomantic-ui.com/modules/toast.html#/settings} */
export type ToastSettings = Partial<FomanticUI.ToastSettings>;
export {};
