/**
 * Form validation rules and types.
 * @module data/validation-rules
 */
import { formDefaults } from "../data/settings";
import { isoDate } from "../data/common";
import { parse } from "../data/helpers";
/*
                              dP                                            dP
                              88                                            88
 .d8888b. dP    dP .d8888b. d8888P .d8888b. 88d8b.d8b.    88d888b. dP    dP 88 .d8888b. .d8888b.
 88'  `"" 88    88 Y8ooooo.   88   88'  `88 88'`88'`88    88'  `88 88    88 88 88ooood8 Y8ooooo.
 88.  ... 88.  .88       88   88   88.  .88 88  88  88    88       88.  .88 88 88.  ...       88
 `88888P' `88888P' `88888P'   dP   `88888P' dP  dP  dP    dP       `88888P' dP `88888P' `88888P'

*/
/** Try to parse date as locale date or as any date */
function tryToParseDate(value) {
    // first try to parse as locale-specific date
    let d = parse.date(value);
    if (!d) {
        // then try JS Date
        try {
            d = new Date(`${value} 13:00`);
        }
        catch (ex) {
            // ignore
        }
    }
    return d;
}
/** Try to parse date as locale date or as any date */
function tryToParseNumber(value) {
    let n = undefined;
    // first try to parse as locale-specific date
    n = parse.num6(value);
    if (n)
        return n;
    n = parse.money(value);
    if (n)
        return n;
    // then try JS number
    n = parseFloat(value);
    if (Number.isNaN(n)) {
        n = undefined;
    }
    return n;
}
/** Returns true if value string represents a date in ISO format */
function isoDateFn(value) {
    let d = undefined;
    try {
        d = new Date(`${value} 13:00`);
    }
    catch (ex) {
        d = undefined;
    }
    return value === isoDate(d);
}
// /** Returns true if value string may be parsed as date */
// function someDateFn(value: string): boolean {
//     const d: Date | undefined = parse.date(value);
//     return value !== undefined;
// }
// /** Returns true if value string represents a date in current locale format */
// function localeDateFn(value: string): boolean {
//     const d: Date | undefined = parse.date(value);
//     const s: string = fmt.date(d);
//     return value === s;
// }
/** Returns true if the string starts with the given value */
function startFn(value, ruleValue) {
    return value.startsWith(ruleValue);
}
/** Returns true if the string starts and ends with the given character */
function wrappedInFn(value, ruleValue) {
    return value.startsWith(ruleValue) && value.endsWith(ruleValue);
}
function greaterOrEqual(v1, v2) {
    return v1 >= v2;
}
function lessOrEqual(v1, v2) {
    return v1 <= v2;
}
/** Return true is value id equal or greater that rule parameter.
    Rule parameter should have type designation like `minValue[D|2015-01-19]` */
function compareTyped(value, ruleValue, compare) {
    if (!value || !ruleValue) {
        return true;
    }
    let ruleVal = ruleValue.trim();
    // compare as string if rule value is wrapped in single quotes
    if (ruleVal.startsWith("'") && ruleVal.endsWith("'")) {
        ruleVal = ruleVal.slice(1, -1);
        return compare(value, ruleVal);
    }
    // compare as date if rule value looks like ISO date
    const isoDateRx = /^\d{4}[-/.]\d{2}[-/.]\d{2}$/;
    const isoDateRule = isoDateRx.test(ruleVal);
    const ruleDate = tryToParseDate(ruleVal);
    if (isoDateRule && ruleDate) {
        const d = tryToParseDate(value);
        return d !== undefined ? compare(d.getTime(), ruleDate.getTime()) : true;
    }
    // compare as number if rule value looks like decimal number
    const numberRx = /^\d*\.?\d*$/;
    const numberRule = numberRx.test(ruleVal);
    const ruleNumber = tryToParseNumber(ruleVal);
    if (numberRule && ruleNumber) {
        const n = tryToParseNumber(value);
        return n !== undefined ? compare(n, ruleNumber) : true;
    }
    // compare as string
    return compare(value, ruleVal);
}
function minValueFn(value, ruleValue) {
    const res = compareTyped(value, ruleValue, greaterOrEqual);
    // formLog.debug("COMPARE :", value, ">=", ruleValue, "=", res);
    return res;
}
function maxValueFn(value, ruleValue) {
    const res = compareTyped(value, ruleValue, lessOrEqual);
    // formLog.debug("COMPARE :", value, "<=", ruleValue, "=", res);
    return res;
}
/*
 oo          oo   dP   oo          dP oo
                  88               88
 dP 88d888b. dP d8888P dP .d8888b. 88 dP d888888b .d8888b.
 88 88'  `88 88   88   88 88'  `88 88 88    .d8P' 88ooood8
 88 88    88 88   88   88 88.  .88 88 88  .Y8P    88.  ...
 dP dP    dP dP   dP   dP `88888P8 dP dP d888888P `88888P'

*/
export function registerRule(name, fn, defaultPrompt) {
    const def = formDefaults.read();
    def.rules[name] = fn;
    def.prompt[name] = defaultPrompt;
}
/** Must be called after DOM is initialized. Like in sveltekit rotes/layout.ts load(). */
export function extendValidationRules() {
    registerRule("start", startFn, "{name} must start with '{ruleValue}'");
    registerRule("isoDate", isoDateFn, "{name} must follow the 'YYYY-MM-DD' format");
    registerRule("wrappedIn", wrappedInFn, "{name} must start and end with '{ruleValue}'");
    registerRule("minValue", minValueFn, "{name} must be greater or equal {ruleValue}");
    registerRule("maxValue", maxValueFn, "{name} must be less or equal {ruleValue}");
}
